﻿#ifndef _LIBKGC_HEADER_H_
#define _LIBKGC_HEADER_H_

#ifdef __cplusplus
extern "C" {
#endif

/* KGC Error Code */
enum {
    /* everything is ok */
    KGC_ERR_NO = 0,
    /* invalid KGC context */
    KGC_ERR_INVALID_CTX,
    /* connect to server timeout */
    KGC_ERR_TIMEOUT,
    /* server reject connect */
    KGC_ERR_REJECT,
    /* send to server error */
    KGC_ERR_SEND,
    /* recv from server error */
    KGC_ERR_RECV,
    /* seq mismatch in c/s */
    KGC_ERR_MISMATCH,
    /* wrong request id */
    KGC_ERR_BAD_REQ,
    /* request mode not support */
    KGC_ERR_BAD_MODE,
    /* error gpu index */
    KGC_ERR_BAD_INDEX,
    /* error args */
    KGC_ERR_BAD_ARGS,
    /* xserver not start */
    KGC_ERR_NO_XSERVER,
    /* xserver KGC not start */
    KGC_ERR_NO_XKGC,
    /* xserver NV-CONTROL not start */
    KGC_ERR_NO_XNVCTRL,
    /* SET not allow due to master */
    KGC_ERR_NOT_MASTER,
    /* alreay has a master */
    KGC_ERR_HAS_MASTER,
    /* request not support */
    KGC_ERR_NOT_SUPPORT,
    /* length is too small */
    KGC_ERR_SMALL_SIZE,
    /* no gou found in system */
    KGC_ERR_NO_GPU,
    /* error count */
    KGC_ERR_LAST,
};

typedef struct _KGC_contex KGC_context;

/* Init and Cleanup function
 * All api need KGC_context arg from KGC_init_local
 * Cleanup context by KGC_cleanup
 */
KGC_context* KGC_init_local();
void KGC_cleanup(KGC_context *ctx);

/* nop is just no */
int KGC_nop(KGC_context *ctx);

/* master is used for set, when set, ctx must be master
 * Only one client can become master at the same time
 * otherwise get -KGC_ERR_NOT_MASTER or -KGC_ERR_HAS_MASTER
 */
int KGC_get_master(KGC_context *ctx, int *master);
int KGC_set_master(KGC_context *ctx);

/* how many graphics card in current system.
 * if zero, there is no reason to continue.
 */
int KGC_get_gpu_number(KGC_context *ctx, int *num);

/* KGC_MAX_NAME_LEN is the default length of name */
#define KGC_MAX_NAME_LEN 128

/* kernel driver name */
int KGC_get_kernel_driver(KGC_context *ctx, int idx, char *name, int size);

/* gpu vender name */
int KGC_get_vendor_name(KGC_context *ctx, int idx, char *name, int size);

/* graphics card name, like RX 5700 */
int KGC_get_card_name(KGC_context *ctx, int idx, char *name, int size);

/* gpu core name */
int KGC_get_gpu_name(KGC_context *ctx, int idx, char *name, int size);

/* pci device info */
int KGC_get_pci_info(KGC_context *ctx, int idx, unsigned int info[5]);

/* gpu core clock SCLK */
int KGC_get_gpu_clock_max(KGC_context *ctx, int idx, unsigned int *clock);

/* vram clock MCLK */
int KGC_get_vram_clock_max(KGC_context *ctx, int idx, unsigned int *clock);

/* vram size */
int KGC_get_vram_size(KGC_context *ctx, int idx, unsigned int *size);

/* pci bus id */
int KGC_get_pci_id(KGC_context *ctx, int idx, char *name, int size);

/* pci bus max link speed*/
int KGC_get_pci_max_link(KGC_context *ctx, int idx, float *link);

/* pci bus width */
int KGC_get_pci_width(KGC_context *ctx, int idx, unsigned int *width);

/* vram type eg. GDDR3 */
int KGC_get_vram_type(KGC_context *ctx, int idx,
                      unsigned int *width, char *name, int size);

/* power manager type */
int KGC_get_power_type(KGC_context *ctx, int idx, char *name, int size);

/* graphics card vbios version */
int KGC_get_vbios_version(KGC_context *ctx, int idx, char *name, int size);

/* gpu shader number */
int KGC_get_shaders_num(KGC_context *ctx, int idx, unsigned int *num);

/* dynamic data */
int KGC_get_gpu_clock(KGC_context *ctx, int idx, unsigned int *clock);

int KGC_get_vram_clock(KGC_context *ctx, int idx, unsigned int *clock);

int KGC_get_gpu_load(KGC_context *ctx, int idx, unsigned int *load);

int KGC_get_vram_load(KGC_context *ctx, int idx, unsigned int *load);

int KGC_get_vram_used(KGC_context *ctx, int idx, unsigned int *used);

int KGC_get_gpu_temp(KGC_context *ctx, int idx, unsigned int *temp);

int KGC_get_pci_speed(KGC_context *ctx, int idx, float *speed, float *version);

/* fan speed type */
#define FAN_SPEED_AUTO 2
#define FAN_SPEED_MANUAL 1
int KGC_get_fan_speed(KGC_context *ctx, int idx, unsigned int *speed);
/* Speed is set to a percentage of fan speed.
 * If fan type is  FAN_SPEED_AUTO, set speed to zero.
 */
int KGC_set_fan_speed(KGC_context *ctx, int idx, int type, int speed);

int KGC_get_video_load(KGC_context *ctx, int idx, unsigned int *load);

int KGC_get_power_load(KGC_context *ctx, int idx, unsigned int *load);

int KGC_get_gpu_vddc(KGC_context *ctx, int idx, unsigned int *vddc);


/*Add asuffix to repalace the API of the same name*/

/* pci bus max link speed/speed_version and width
 * eg.
 *   bus_speed : 0 GT/s (0.0)
 *   bus_interface : PCIe x 8
*/
int KGC_get_pci_max_link_speed(KGC_context *ctx, int idx, float *link, float *version);
int KGC_get_pci_max_link_width(KGC_context *ctx, int idx, unsigned int *width);

int KGC_get_gpu_temp_new(KGC_context *ctx, int idx, float *temp);

int KGC_get_gpu_load_new(KGC_context *ctx, int idx, float *load);

int KGC_get_vram_load_new(KGC_context *ctx, int idx, float *load);

int KGC_get_power_load_new(KGC_context *ctx, int idx, float *load);

int KGC_get_video_load_new(KGC_context *ctx, int idx, float *load);

/* Replace function: KGC_get_pci_width
 * current pci bus width
*/
int KGC_get_cur_pci_width(KGC_context *ctx, int idx, unsigned int *width);

/* Replace function: KGC_get_pci_speed
 * unit : %
*/
int KGC_get_cur_pci_speed(KGC_context *ctx, int idx, float *speed);


/* power setting*/
int KGC_get_current_power_mode(KGC_context *ctx, int idx, char *mode_name, int size);

/*power mizer value and mode name
 * mode_name0
 * mode_name1
 */
int KGC_get_power_mode_list(KGC_context *ctx, int idx, char *mode, int size);

int KGC_set_power_mode(KGC_context *ctx, int idx, int mode_value);

/* gpu threshold type */
typedef enum {
    THR_TYPE_TEMP = 0,
    THR_TYPE_LOAD,
    THR_TYPE_POWER,
    /* add more */
    THR_TYPE_NUM,
} thr_type;

#define THR_MASK_TEMP   (1 << THR_TYPE_TEMP)
#define THR_MASK_LOAD   (1 << THR_TYPE_LOAD)
#define THR_MASK_POWER  (1 << THR_TYPE_POWER)
#define THR_MASK_ALL    (~(~0 << THR_TYPE_NUM))

/* get and set threshold */
int KGC_get_threshold_value(KGC_context *ctx, int idx,
                            thr_type type, unsigned int *thr);

int KGC_set_threshold_value(KGC_context *ctx, int idx,
                            thr_type type, unsigned int thr);

/* enable or disable gpu monitor threshod
 * mask bitfiled == 0 means disabled
 */
int KGC_get_threshod_mask(KGC_context *ctx, int idx, unsigned int *mask);

int KGC_set_threshod_mask(KGC_context *ctx, int idx, unsigned int mask);


/* software information */

/* 2D driver name in XServer */
int KGC_get_user_driver(KGC_context *ctx, int idx, char *name, int size);
/* 2D driver accel method in XServer */
int KGC_get_accel_method(KGC_context *ctx, int idx, char *name, int size);

/* api info */
int KGC_get_egl_client(KGC_context *ctx, char *data, int size);
int KGC_get_egl_vendor(KGC_context *ctx, char *data, int size);
int KGC_get_egl_version(KGC_context *ctx, char *data, int size);
int KGC_get_egl_extension(KGC_context *ctx, char *data, int size);

int KGC_get_glx_extension(KGC_context *ctx, char *data, int size);
int KGC_get_glx_client_extension(KGC_context *ctx, char *data, int size);
int KGC_get_glx_client_vendor(KGC_context *ctx, char *data, int size);
int KGC_get_glx_client_version(KGC_context *ctx, char *data, int size);
int KGC_get_glx_server_extension(KGC_context *ctx, char *data, int size);
int KGC_get_glx_server_vendor(KGC_context *ctx, char *data, int size);
int KGC_get_glx_server_version(KGC_context *ctx, char *data, int size);

int KGC_get_opengl_extension(KGC_context *ctx, char *data, int size);
int KGC_get_opengl_vendor(KGC_context *ctx, char *data, int size);
int KGC_get_opengl_version(KGC_context *ctx, char *data, int size);
int KGC_get_opengl_SL_version(KGC_context *ctx, char *data, int size);
int KGC_get_opengl_direct(KGC_context *ctx, char *data, int size);
int KGC_get_opengl_renderer(KGC_context *ctx, char *data, int size);

int KGC_get_opengl_core_ver(KGC_context *ctx, char *data, int size);
int KGC_get_opengl_core_SL_ver(KGC_context *ctx, char *data, int size);
int KGC_get_opengl_core_extension(KGC_context *ctx, char *data, int size);

int KGC_get_opengl_ES_ver(KGC_context *ctx, char *data, int size);
int KGC_get_opengl_ES_SL_ver(KGC_context *ctx, char *data, int size);
int KGC_get_opengl_ES_extension(KGC_context *ctx, char *data, int size);

int KGC_get_vaapi_driver_version(KGC_context *ctx, char *data, int size);
int KGC_get_vaapi_support(KGC_context *ctx, char *data, int size);
int KGC_get_vaapi_version(KGC_context *ctx, char *data, int size);

int KGC_get_vdpau_base_info(KGC_context *ctx, char *data, int size);
int KGC_get_vdpau_version(KGC_context *ctx, unsigned int *nums);

int KGC_get_cuda_version(KGC_context *ctx, int idx, char *data, int size);

int KGC_get_vulkan_instance_version(KGC_context *ctx, char *data, int size);
int KGC_get_vulkan_dev_support(KGC_context *ctx, char *data, int size);
int KGC_get_vulkan_instance_layer(KGC_context *ctx, char *data, int size);
int KGC_get_vulkan_instance_extension(KGC_context *ctx, char *data, int size);

int KGC_get_opencl_platform_nums(KGC_context *ctx, unsigned int *nums);
int KGC_get_opencl_platform_info(KGC_context *ctx, char *data, int size);
int KGC_get_opencl_device_info(KGC_context *ctx, char *data, int size);


/* To use your discrete card for the applications who need it the most (for example games, 3D modellers...) */

int KGC_get_source_output(KGC_context *ctx, int *index);
/**
 * @brief KGC_get_offload get environment variables.You can add environment variables to run the program
 * @param data : environment variables
 */
int KGC_get_offload_env(KGC_context *ctx, int index, char *data, int size);

#ifdef __cplusplus
}
#endif

#endif /* _LIBKGC_HEADER_H_ */
